@@ -29,6 +29,7 @@
 #include <pthread.h>
 #include <signal.h>
 #include <stddef.h>
+#include <stdint.h>
 #include <sys/stat.h>
 #include <unistd.h>
 
@@ -609,103 +610,6 @@ static unsigned int lttng_sessions_count(uid_t uid, gid_t gid)
 	return i;
 }
 
-static int receive_userspace_probe(struct command_ctx *cmd_ctx, int sock,
-		int *sock_error, struct lttng_event *event)
-{
-	int fd = -1, ret;
-	struct lttng_userspace_probe_location *probe_location;
-	struct lttng_payload probe_location_payload;
-	struct fd_handle *handle = NULL;
-
-	/*
-	 * Create a payload to store the serialized version of the probe
-	 * location.
-	 */
-	lttng_payload_init(&probe_location_payload);
-
-	ret = lttng_dynamic_buffer_set_size(&probe_location_payload.buffer,
-			cmd_ctx->lsm.u.enable.userspace_probe_location_len);
-	if (ret) {
-		ret = LTTNG_ERR_NOMEM;
-		goto error;
-	}
-
-	/*
-	 * Receive the probe location.
-	 */
-	ret = lttcomm_recv_unix_sock(sock, probe_location_payload.buffer.data,
-			probe_location_payload.buffer.size);
-	if (ret <= 0) {
-		DBG("Nothing recv() from client var len data... continuing");
-		*sock_error = 1;
-		ret = LTTNG_ERR_PROBE_LOCATION_INVAL;
-		goto error;
-	}
-
-	/*
-	 * Receive the file descriptor to the target binary from the client.
-	 */
-	DBG("Receiving userspace probe target FD from client ...");
-	ret = lttcomm_recv_fds_unix_sock(sock, &fd, 1);
-	if (ret <= 0) {
-		DBG("Nothing recv() from client userspace probe fd... continuing");
-		*sock_error = 1;
-		ret = LTTNG_ERR_PROBE_LOCATION_INVAL;
-		goto error;
-	}
-
-	handle = fd_handle_create(fd);
-	if (!handle) {
-		ret = LTTNG_ERR_NOMEM;
-		goto error;
-	}
-
-	/* Transferred to the handle. */
-	fd = -1;
-
-	ret = lttng_payload_push_fd_handle(&probe_location_payload, handle);
-	if (ret) {
-		ERR("Failed to add userspace probe file descriptor to payload");
-		ret = LTTNG_ERR_NOMEM;
-		goto error;
-	}
-
-	fd_handle_put(handle);
-	handle = NULL;
-
-	{
-		struct lttng_payload_view view = lttng_payload_view_from_payload(
-			&probe_location_payload, 0, -1);
-
-		/* Extract the probe location from the serialized version. */
-		ret = lttng_userspace_probe_location_create_from_payload(
-				&view, &probe_location);
-	}
-	if (ret < 0) {
-		WARN("Failed to create a userspace probe location from the received buffer");
-		ret = LTTNG_ERR_PROBE_LOCATION_INVAL;
-		goto error;
-	}
-
-	/* Attach the probe location to the event. */
-	ret = lttng_event_set_userspace_probe_location(event, probe_location);
-	if (ret) {
-		ret = LTTNG_ERR_PROBE_LOCATION_INVAL;
-		goto error;
-	}
-
-error:
-	if (fd >= 0) {
-		if (close(fd)) {
-			PERROR("Failed to close userspace probe location binary fd");
-		}
-	}
-
-	fd_handle_put(handle);
-	lttng_payload_reset(&probe_location_payload);
-	return ret;
-}
-
 static enum lttng_error_code receive_lttng_trigger(struct command_ctx *cmd_ctx,
 		int sock,
 		int *sock_error,
@@ -850,6 +754,178 @@ end:
 	return ret_code;
 }
 
+static enum lttng_error_code receive_lttng_event(struct command_ctx *cmd_ctx,
+		int sock,
+		int *sock_error,
+		struct lttng_event **out_event,
+		char **out_filter_expression,
+		struct lttng_bytecode **out_bytecode,
+		struct lttng_event_exclusion **out_exclusion)
+{
+	int ret;
+	size_t event_len;
+	ssize_t sock_recv_len;
+	enum lttng_error_code ret_code;
+	struct lttng_payload event_payload;
+	struct lttng_event *local_event = NULL;
+	char *local_filter_expression = NULL;
+	struct lttng_bytecode *local_bytecode = NULL;
+	struct lttng_event_exclusion *local_exclusion = NULL;
+
+	lttng_payload_init(&event_payload);
+	if (cmd_ctx->lsm.cmd_type == LTTNG_ENABLE_EVENT) {
+		event_len = (size_t) cmd_ctx->lsm.u.enable.length;
+	} else if (cmd_ctx->lsm.cmd_type == LTTNG_DISABLE_EVENT) {
+		event_len = (size_t) cmd_ctx->lsm.u.disable.length;
+	} else {
+		abort();
+	}
+
+	ret = lttng_dynamic_buffer_set_size(&event_payload.buffer, event_len);
+	if (ret) {
+		ret_code = LTTNG_ERR_NOMEM;
+		goto end;
+	}
+
+	sock_recv_len = lttcomm_recv_unix_sock(
+			sock, event_payload.buffer.data, event_len);
+	if (sock_recv_len < 0 || sock_recv_len != event_len) {
+		ERR("Failed to receive event in command payload");
+		*sock_error = 1;
+		ret_code = LTTNG_ERR_INVALID_PROTOCOL;
+		goto end;
+	}
+
+	/* Receive fds, if any. */
+	if (cmd_ctx->lsm.fd_count > 0) {
+		sock_recv_len = lttcomm_recv_payload_fds_unix_sock(
+				sock, cmd_ctx->lsm.fd_count, &event_payload);
+		if (sock_recv_len > 0 &&
+				sock_recv_len != cmd_ctx->lsm.fd_count * sizeof(int)) {
+			ERR("Failed to receive all file descriptors for event in command payload: expected fd count = %u, ret = %d",
+					cmd_ctx->lsm.fd_count, (int) ret);
+			ret_code = LTTNG_ERR_INVALID_PROTOCOL;
+			*sock_error = 1;
+			goto end;
+		} else if (sock_recv_len <= 0) {
+			ERR("Failed to receive file descriptors for event in command payload: expected fd count = %u, ret = %d",
+					cmd_ctx->lsm.fd_count, (int) ret);
+			ret_code = LTTNG_ERR_FATAL;
+			*sock_error = 1;
+			goto end;
+		}
+	}
+
+	/* Deserialize event. */
+	{
+		ssize_t len;
+		struct lttng_payload_view event_view =
+				lttng_payload_view_from_payload(
+						&event_payload, 0, -1);
+
+		len = lttng_event_create_from_payload(&event_view, &local_event,
+				    &local_exclusion, &local_filter_expression,
+				    &local_bytecode);
+
+		if (len < 0) {
+			ERR("Failed to create an event from the received buffer");
+			ret_code = LTTNG_ERR_INVALID_PROTOCOL;
+			goto end;
+		}
+
+		if (len != event_len) {
+			ERR("Userspace probe location from the received buffer is not the advertised length: header length = %zu" PRIu32 ", payload length = %zd", event_len, len);
+			ret_code = LTTNG_ERR_INVALID_PROTOCOL;
+			goto end;
+		}
+	}
+
+	*out_event = local_event;
+	*out_exclusion = local_exclusion;
+	*out_filter_expression = local_filter_expression;
+	*out_bytecode = local_bytecode;
+	local_event = NULL;
+	local_exclusion = NULL;
+	local_filter_expression = NULL;
+	local_bytecode = NULL;
+
+	ret_code = LTTNG_OK;
+
+end:
+	lttng_payload_reset(&event_payload);
+	lttng_event_destroy(local_event);
+	free(local_filter_expression);
+	free(local_bytecode);
+	free(local_exclusion);
+	return ret_code;
+}
+
+static enum lttng_error_code receive_lttng_event_context(
+		const struct command_ctx *cmd_ctx,
+		int sock,
+		int *sock_error,
+		struct lttng_event_context **out_event_context)
+{
+	int ret;
+	const size_t event_context_len =
+			(size_t) cmd_ctx->lsm.u.context.length;
+	ssize_t sock_recv_len;
+	enum lttng_error_code ret_code;
+	struct lttng_payload event_context_payload;
+	struct lttng_event_context *context = NULL;
+
+	lttng_payload_init(&event_context_payload);
+
+	ret = lttng_dynamic_buffer_set_size(&event_context_payload.buffer,
+			event_context_len);
+	if (ret) {
+		ret_code = LTTNG_ERR_NOMEM;
+		goto end;
+	}
+
+	sock_recv_len = lttcomm_recv_unix_sock(
+			sock, event_context_payload.buffer.data,
+			event_context_len);
+	if (sock_recv_len < 0 || sock_recv_len != event_context_len) {
+		ERR("Failed to receive event context in command payload");
+		*sock_error = 1;
+		ret_code = LTTNG_ERR_INVALID_PROTOCOL;
+		goto end;
+	}
+
+	/* Deserialize event. */
+	{
+		ssize_t len;
+		struct lttng_payload_view event_context_view =
+				lttng_payload_view_from_payload(
+						&event_context_payload, 0, -1);
+
+		len = lttng_event_context_create_from_payload(
+				&event_context_view, &context);
+
+		if (len < 0) {
+			ERR("Failed to create a event context from the received buffer");
+			ret_code = LTTNG_ERR_INVALID_PROTOCOL;
+			goto end;
+		}
+
+		if (len != event_context_len) {
+			ERR("Event context from the received buffer is not the advertised length: expected length = %zu, payload length = %zd", event_context_len, len);
+			ret_code = LTTNG_ERR_INVALID_PROTOCOL;
+			goto end;
+		}
+	}
+
+	*out_event_context = context;
+	context = NULL;
+	ret_code = LTTNG_OK;
+
+end:
+	lttng_event_context_destroy(context);
+	lttng_payload_reset(&event_context_payload);
+	return ret_code;
+}
+
 /*
  * Version of setup_lttng_msg() without command header.
  */
@@ -1323,74 +1399,18 @@ skip_domain:
 	switch (cmd_ctx->lsm.cmd_type) {
 	case LTTNG_ADD_CONTEXT:
 	{
-		/*
-		 * An LTTNG_ADD_CONTEXT command might have a supplementary
-		 * payload if the context being added is an application context.
-		 */
-		if (cmd_ctx->lsm.u.context.ctx.ctx ==
-				LTTNG_EVENT_CONTEXT_APP_CONTEXT) {
-			char *provider_name = NULL, *context_name = NULL;
-			size_t provider_name_len =
-					cmd_ctx->lsm.u.context.provider_name_len;
-			size_t context_name_len =
-					cmd_ctx->lsm.u.context.context_name_len;
+		struct lttng_event_context *event_context = NULL;
+		const enum lttng_error_code ret_code =
+			receive_lttng_event_context(
+				cmd_ctx, *sock, sock_error, &event_context);
 
-			if (provider_name_len == 0 || context_name_len == 0) {
-				/*
-				 * Application provider and context names MUST
-				 * be provided.
-				 */
-				ret = -LTTNG_ERR_INVALID;
-				goto error;
-			}
-
-			provider_name = zmalloc(provider_name_len + 1);
-			if (!provider_name) {
-				ret = -LTTNG_ERR_NOMEM;
-				goto error;
-			}
-			cmd_ctx->lsm.u.context.ctx.u.app_ctx.provider_name =
-					provider_name;
-
-			context_name = zmalloc(context_name_len + 1);
-			if (!context_name) {
-				ret = -LTTNG_ERR_NOMEM;
-				goto error_add_context;
-			}
-			cmd_ctx->lsm.u.context.ctx.u.app_ctx.ctx_name =
-					context_name;
-
-			ret = lttcomm_recv_unix_sock(*sock, provider_name,
-					provider_name_len);
-			if (ret < 0) {
-				goto error_add_context;
-			}
-
-			ret = lttcomm_recv_unix_sock(*sock, context_name,
-					context_name_len);
-			if (ret < 0) {
-				goto error_add_context;
-			}
-		}
-
-		/*
-		 * cmd_add_context assumes ownership of the provider and context
-		 * names.
-		 */
-		ret = cmd_add_context(cmd_ctx->session,
-				cmd_ctx->lsm.domain.type,
-				cmd_ctx->lsm.u.context.channel_name,
-				ALIGNED_CONST_PTR(cmd_ctx->lsm.u.context.ctx),
-				the_kernel_poll_pipe[1]);
-
-		cmd_ctx->lsm.u.context.ctx.u.app_ctx.provider_name = NULL;
-		cmd_ctx->lsm.u.context.ctx.u.app_ctx.ctx_name = NULL;
-error_add_context:
-		free(cmd_ctx->lsm.u.context.ctx.u.app_ctx.provider_name);
-		free(cmd_ctx->lsm.u.context.ctx.u.app_ctx.ctx_name);
-		if (ret < 0) {
+		if (ret_code != LTTNG_OK) {
+			ret = (int) ret_code;
 			goto error;
 		}
+
+		ret = cmd_add_context(cmd_ctx, event_context, the_kernel_poll_pipe[1]);
+		lttng_event_context_destroy(event_context);
 		break;
 	}
 	case LTTNG_DISABLE_CHANNEL:
@@ -1399,48 +1419,10 @@ error_add_context:
 				cmd_ctx->lsm.u.disable.channel_name);
 		break;
 	}
-	case LTTNG_DISABLE_EVENT:
-	{
-
-		/*
-		 * FIXME: handle filter; for now we just receive the filter's
-		 * bytecode along with the filter expression which are sent by
-		 * liblttng-ctl and discard them.
-		 *
-		 * This fixes an issue where the client may block while sending
-		 * the filter payload and encounter an error because the session
-		 * daemon closes the socket without ever handling this data.
-		 */
-		size_t count = cmd_ctx->lsm.u.disable.expression_len +
-			cmd_ctx->lsm.u.disable.bytecode_len;
-
-		if (count) {
-			char data[LTTNG_FILTER_MAX_LEN];
-
-			DBG("Discarding disable event command payload of size %zu", count);
-			while (count) {
-				ret = lttcomm_recv_unix_sock(*sock, data,
-				        count > sizeof(data) ? sizeof(data) : count);
-				if (ret < 0) {
-					goto error;
-				}
-
-				count -= (size_t) ret;
-			}
-		}
-		ret = cmd_disable_event(cmd_ctx->session, cmd_ctx->lsm.domain.type,
-				cmd_ctx->lsm.u.disable.channel_name,
-				ALIGNED_CONST_PTR(cmd_ctx->lsm.u.disable.event));
-		break;
-	}
 	case LTTNG_ENABLE_CHANNEL:
 	{
-		cmd_ctx->lsm.u.channel.chan.attr.extended.ptr =
-				(struct lttng_channel_extended *) &cmd_ctx->lsm.u.channel.extended;
-		ret = cmd_enable_channel(cmd_ctx->session,
-				ALIGNED_CONST_PTR(cmd_ctx->lsm.domain),
-				ALIGNED_CONST_PTR(cmd_ctx->lsm.u.channel.chan),
-				the_kernel_poll_pipe[1]);
+		ret = cmd_enable_channel(
+				cmd_ctx, *sock, the_kernel_poll_pipe[1]);
 		break;
 	}
 	case LTTNG_PROCESS_ATTR_TRACKER_ADD_INCLUDE_VALUE:
@@ -1642,224 +1624,122 @@ error_add_context:
 		break;
 	}
 	case LTTNG_ENABLE_EVENT:
+	case LTTNG_DISABLE_EVENT:
 	{
-		struct lttng_event *ev = NULL;
-		struct lttng_event_exclusion *exclusion = NULL;
-		struct lttng_bytecode *bytecode = NULL;
-		char *filter_expression = NULL;
+		struct lttng_event *event;
+		char *filter_expression;
+		struct lttng_event_exclusion *exclusions;
+		struct lttng_bytecode *bytecode;
+		const enum lttng_error_code ret_code = receive_lttng_event(
+				cmd_ctx, *sock, sock_error, &event,
+				&filter_expression, &bytecode, &exclusions);
 
-		/* Handle exclusion events and receive it from the client. */
-		if (cmd_ctx->lsm.u.enable.exclusion_count > 0) {
-			size_t count = cmd_ctx->lsm.u.enable.exclusion_count;
-
-			exclusion = zmalloc(sizeof(struct lttng_event_exclusion) +
-					(count * LTTNG_SYMBOL_NAME_LEN));
-			if (!exclusion) {
-				ret = LTTNG_ERR_EXCLUSION_NOMEM;
-				goto error;
-			}
-
-			DBG("Receiving var len exclusion event list from client ...");
-			exclusion->count = count;
-			ret = lttcomm_recv_unix_sock(*sock, exclusion->names,
-					count * LTTNG_SYMBOL_NAME_LEN);
-			if (ret <= 0) {
-				DBG("Nothing recv() from client var len data... continuing");
-				*sock_error = 1;
-				free(exclusion);
-				ret = LTTNG_ERR_EXCLUSION_INVAL;
-				goto error;
-			}
-		}
-
-		/* Get filter expression from client. */
-		if (cmd_ctx->lsm.u.enable.expression_len > 0) {
-			size_t expression_len =
-				cmd_ctx->lsm.u.enable.expression_len;
-
-			if (expression_len > LTTNG_FILTER_MAX_LEN) {
-				ret = LTTNG_ERR_FILTER_INVAL;
-				free(exclusion);
-				goto error;
-			}
-
-			filter_expression = zmalloc(expression_len);
-			if (!filter_expression) {
-				free(exclusion);
-				ret = LTTNG_ERR_FILTER_NOMEM;
-				goto error;
-			}
-
-			/* Receive var. len. data */
-			DBG("Receiving var len filter's expression from client ...");
-			ret = lttcomm_recv_unix_sock(*sock, filter_expression,
-				expression_len);
-			if (ret <= 0) {
-				DBG("Nothing recv() from client var len data... continuing");
-				*sock_error = 1;
-				free(filter_expression);
-				free(exclusion);
-				ret = LTTNG_ERR_FILTER_INVAL;
-				goto error;
-			}
-		}
-
-		/* Handle filter and get bytecode from client. */
-		if (cmd_ctx->lsm.u.enable.bytecode_len > 0) {
-			size_t bytecode_len = cmd_ctx->lsm.u.enable.bytecode_len;
-
-			if (bytecode_len > LTTNG_FILTER_MAX_LEN) {
-				ret = LTTNG_ERR_FILTER_INVAL;
-				free(filter_expression);
-				free(exclusion);
-				goto error;
-			}
-
-			bytecode = zmalloc(bytecode_len);
-			if (!bytecode) {
-				free(filter_expression);
-				free(exclusion);
-				ret = LTTNG_ERR_FILTER_NOMEM;
-				goto error;
-			}
-
-			/* Receive var. len. data */
-			DBG("Receiving var len filter's bytecode from client ...");
-			ret = lttcomm_recv_unix_sock(*sock, bytecode, bytecode_len);
-			if (ret <= 0) {
-				DBG("Nothing recv() from client var len data... continuing");
-				*sock_error = 1;
-				free(filter_expression);
-				free(bytecode);
-				free(exclusion);
-				ret = LTTNG_ERR_FILTER_INVAL;
-				goto error;
-			}
-
-			if ((bytecode->len + sizeof(*bytecode)) != bytecode_len) {
-				free(filter_expression);
-				free(bytecode);
-				free(exclusion);
-				ret = LTTNG_ERR_FILTER_INVAL;
-				goto error;
-			}
-		}
-
-		ev = lttng_event_copy(ALIGNED_CONST_PTR(cmd_ctx->lsm.u.enable.event));
-		if (!ev) {
-			DBG("Failed to copy event: %s",
-					cmd_ctx->lsm.u.enable.event.name);
-			free(filter_expression);
-			free(bytecode);
-			free(exclusion);
-			ret = LTTNG_ERR_NOMEM;
-			goto error;
-		}
-
-
-		if (cmd_ctx->lsm.u.enable.userspace_probe_location_len > 0) {
-			/* Expect a userspace probe description. */
-			ret = receive_userspace_probe(cmd_ctx, *sock, sock_error, ev);
-			if (ret) {
-				free(filter_expression);
-				free(bytecode);
-				free(exclusion);
-				lttng_event_destroy(ev);
-				goto error;
-			}
-		}
-
-		ret = cmd_enable_event(cmd_ctx->session,
-				ALIGNED_CONST_PTR(cmd_ctx->lsm.domain),
-				cmd_ctx->lsm.u.enable.channel_name,
-				ev,
-				filter_expression, bytecode, exclusion,
-				the_kernel_poll_pipe[1]);
-		lttng_event_destroy(ev);
-		break;
-	}
-	case LTTNG_LIST_TRACEPOINTS:
-	{
-		struct lttng_event *events;
-		ssize_t nb_events;
-
-		session_lock_list();
-		nb_events = cmd_list_tracepoints(cmd_ctx->lsm.domain.type, &events);
-		session_unlock_list();
-		if (nb_events < 0) {
-			/* Return value is a negative lttng_error_code. */
-			ret = -nb_events;
+		if (ret_code != LTTNG_OK) {
+			ret = (int) ret_code;
 			goto error;
 		}
 
 		/*
-		 * Setup lttng message with payload size set to the event list size in
-		 * bytes and then copy list into the llm payload.
+		 * Ownership of filter_expression, exclusions, and bytecode is
+		 * always transferred.
 		 */
-		ret = setup_lttng_msg_no_cmd_header(cmd_ctx, events,
-			sizeof(struct lttng_event) * nb_events);
-		free(events);
+		ret = cmd_ctx->lsm.cmd_type == LTTNG_ENABLE_EVENT ?
+				cmd_enable_event(cmd_ctx, event,
+						filter_expression, exclusions,
+						bytecode,
+						the_kernel_poll_pipe[1]) :
+				cmd_disable_event(cmd_ctx, event,
+						filter_expression, bytecode,
+						exclusions);
+		lttng_event_destroy(event);
+		break;
+	}
+	case LTTNG_LIST_TRACEPOINTS:
+	{
+		enum lttng_error_code ret_code;
+		size_t original_payload_size;
+		size_t payload_size;
+		const size_t command_header_size = sizeof(struct lttcomm_list_command_header);
 
-		if (ret < 0) {
+		ret = setup_empty_lttng_msg(cmd_ctx);
+		if (ret) {
+			ret = LTTNG_ERR_NOMEM;
 			goto setup_error;
 		}
 
+		original_payload_size = cmd_ctx->reply_payload.buffer.size;
+
+		session_lock_list();
+		ret_code = cmd_list_tracepoints(cmd_ctx->lsm.domain.type,
+				&cmd_ctx->reply_payload);
+		session_unlock_list();
+		if (ret_code != LTTNG_OK) {
+			ret = (int) ret_code;
+			goto error;
+		}
+
+		payload_size = cmd_ctx->reply_payload.buffer.size -
+				command_header_size - original_payload_size;
+		update_lttng_msg(cmd_ctx, command_header_size, payload_size);
+
 		ret = LTTNG_OK;
 		break;
 	}
 	case LTTNG_LIST_TRACEPOINT_FIELDS:
 	{
-		struct lttng_event_field *fields;
-		ssize_t nb_fields;
+		enum lttng_error_code ret_code;
+		size_t original_payload_size;
+		size_t payload_size;
+		const size_t command_header_size = sizeof(struct lttcomm_list_command_header);
+
+		ret = setup_empty_lttng_msg(cmd_ctx);
+		if (ret) {
+			ret = LTTNG_ERR_NOMEM;
+			goto setup_error;
+		}
+
+		original_payload_size = cmd_ctx->reply_payload.buffer.size;
 
 		session_lock_list();
-		nb_fields = cmd_list_tracepoint_fields(cmd_ctx->lsm.domain.type,
-				&fields);
+		ret_code = cmd_list_tracepoint_fields(
+				cmd_ctx->lsm.domain.type, &cmd_ctx->reply_payload);
 		session_unlock_list();
-		if (nb_fields < 0) {
-			/* Return value is a negative lttng_error_code. */
-			ret = -nb_fields;
+		if (ret_code != LTTNG_OK) {
+			ret = (int) ret_code;
 			goto error;
 		}
 
-		/*
-		 * Setup lttng message with payload size set to the event list size in
-		 * bytes and then copy list into the llm payload.
-		 */
-		ret = setup_lttng_msg_no_cmd_header(cmd_ctx, fields,
-				sizeof(struct lttng_event_field) * nb_fields);
-		free(fields);
-
-		if (ret < 0) {
-			goto setup_error;
-		}
+		payload_size = cmd_ctx->reply_payload.buffer.size -
+				command_header_size - original_payload_size;
+		update_lttng_msg(cmd_ctx, command_header_size, payload_size);
 
 		ret = LTTNG_OK;
 		break;
 	}
 	case LTTNG_LIST_SYSCALLS:
 	{
-		struct lttng_event *events;
-		ssize_t nb_events;
+		enum lttng_error_code ret_code;
+		size_t original_payload_size;
+		size_t payload_size;
+		const size_t command_header_size = sizeof(struct lttcomm_list_command_header);
 
-		nb_events = cmd_list_syscalls(&events);
-		if (nb_events < 0) {
-			/* Return value is a negative lttng_error_code. */
-			ret = -nb_events;
+		ret = setup_empty_lttng_msg(cmd_ctx);
+		if (ret) {
+			ret = LTTNG_ERR_NOMEM;
+			goto setup_error;
+		}
+
+		original_payload_size = cmd_ctx->reply_payload.buffer.size;
+
+		ret_code = cmd_list_syscalls(&cmd_ctx->reply_payload);
+		if (ret_code != LTTNG_OK) {
+			ret = (int) ret_code;
 			goto error;
 		}
 
-		/*
-		 * Setup lttng message with payload size set to the event list size in
-		 * bytes and then copy list into the llm payload.
-		 */
-		ret = setup_lttng_msg_no_cmd_header(cmd_ctx, events,
-			sizeof(struct lttng_event) * nb_events);
-		free(events);
-
-		if (ret < 0) {
-			goto setup_error;
-		}
+		payload_size = cmd_ctx->reply_payload.buffer.size -
+				command_header_size - original_payload_size;
+		update_lttng_msg(cmd_ctx, command_header_size, payload_size);
 
 		ret = LTTNG_OK;
 		break;
@@ -1958,34 +1838,10 @@ error_add_context:
 	}
 	case LTTNG_LIST_CHANNELS:
 	{
-		ssize_t payload_size;
-		struct lttng_channel *channels = NULL;
-
-		payload_size = cmd_list_channels(cmd_ctx->lsm.domain.type,
-				cmd_ctx->session, &channels);
-		if (payload_size < 0) {
-			/* Return value is a negative lttng_error_code. */
-			ret = -payload_size;
-			goto error;
-		}
-
-		ret = setup_lttng_msg_no_cmd_header(cmd_ctx, channels,
-			payload_size);
-		free(channels);
-
-		if (ret < 0) {
-			goto setup_error;
-		}
-
-		ret = LTTNG_OK;
-		break;
-	}
-	case LTTNG_LIST_EVENTS:
-	{
-		ssize_t list_ret;
-		struct lttcomm_event_command_header cmd_header = {};
+		enum lttng_error_code ret_code;
 		size_t original_payload_size;
 		size_t payload_size;
+		const size_t command_header_size = sizeof(struct lttcomm_list_command_header);
 
 		ret = setup_empty_lttng_msg(cmd_ctx);
 		if (ret) {
@@ -1995,20 +1851,46 @@ error_add_context:
 
 		original_payload_size = cmd_ctx->reply_payload.buffer.size;
 
-		/* Extended infos are included at the end of the payload. */
-		list_ret = cmd_list_events(cmd_ctx->lsm.domain.type,
-				cmd_ctx->session,
-				cmd_ctx->lsm.u.list.channel_name,
-				&cmd_ctx->reply_payload);
-		if (list_ret < 0) {
-			/* Return value is a negative lttng_error_code. */
-			ret = -list_ret;
+		ret_code = cmd_list_channels(cmd_ctx->lsm.domain.type,
+				cmd_ctx->session, &cmd_ctx->reply_payload);
+		if (ret_code != LTTNG_OK) {
+			ret = (int) ret_code;
 			goto error;
 		}
 
 		payload_size = cmd_ctx->reply_payload.buffer.size -
-				sizeof(cmd_header) - original_payload_size;
-		update_lttng_msg(cmd_ctx, sizeof(cmd_header), payload_size);
+				command_header_size - original_payload_size;
+		update_lttng_msg(cmd_ctx, command_header_size, payload_size);
+
+		ret = LTTNG_OK;
+		break;
+	}
+	case LTTNG_LIST_EVENTS:
+	{
+		enum lttng_error_code ret_code;
+		size_t original_payload_size;
+		size_t payload_size;
+		const size_t command_header_size = sizeof(struct lttcomm_list_command_header);
+
+		ret = setup_empty_lttng_msg(cmd_ctx);
+		if (ret) {
+			ret = LTTNG_ERR_NOMEM;
+			goto setup_error;
+		}
+
+		original_payload_size = cmd_ctx->reply_payload.buffer.size;
+
+		ret_code = cmd_list_events(cmd_ctx->lsm.domain.type,
+				cmd_ctx->session,
+				cmd_ctx->lsm.u.list.channel_name, &cmd_ctx->reply_payload);
+		if (ret_code != LTTNG_OK) {
+			ret = (int) ret_code;
+			goto error;
+		}
+
+		payload_size = cmd_ctx->reply_payload.buffer.size -
+				command_header_size - original_payload_size;
+		update_lttng_msg(cmd_ctx, command_header_size, payload_size);
 
 		ret = LTTNG_OK;
 		break;
